@@ -15,20 +15,25 @@
 # You should have received a copy of the GNU General Public License along
 # with this program; if not, write to the Free Software Foundation, Inc.,
 # 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA.
-    
+
 
 ## helper to parse parameter lists with defaults
+.nodots <- function(...) {
+  l <- list(...)
+  if(length(l) > 0L) warning("Unknown arguments: ",
+    paste(names(l), "=",l, collapse=", "))
+}
 
 .get_parameters <- function(parameter, defaults) {
   defaults <- as.list(defaults)
   parameter <- as.list(parameter)
-  
+
   ## add verbose
   if(is.null(defaults$verbose)) defaults$verbose <- FALSE
-  
+
   if(length(parameter) != 0) {
     o <- pmatch(names(parameter), names(defaults))
-    
+
     ## unknown parameter
     if(any(is.na(o))){
       warning(sprintf(ngettext(length(is.na(o)),
@@ -36,22 +41,24 @@
         "Unknown parameters: %s"),
         paste(names(parameter)[is.na(o)],
           collapse = ", ")), call. = FALSE, immediate. = TRUE)
-      
+
       cat("Available parameter (with default values):\n")
       #print(defaults)
-      cat(rbind(names(defaults)," = ", gsub("\n"," ",as.character(defaults))), 
+      cat(rbind(names(defaults)," = ", gsub("\n"," ",as.character(defaults))),
         sep=c("\t"," ","\n"))
-    } 
-    
+    }
+
     defaults[o[!is.na(o)]] <- parameter[!is.na(o)]
   }
 
   if(defaults$verbose) {
     cat("Used parameters:\n")
     #print(defaults)
-    cat(rbind(names(defaults)," = ", gsub("\n"," ",as.character(defaults))), 
+    cat(rbind(names(defaults)," = ",
+      strtrim(gsub("\n"," ",as.character(defaults)), 50)),
       sep=c("\t"," ","\n"))
   }
-  
+
   defaults
 }
+
