@@ -20,24 +20,24 @@
 ## matrix
 
 criterion.dist <- function(x, order = NULL, method = NULL, ...) {
-    
+
     ## check order and x
     if(!is.null(order)) {
         if(!inherits(order, "ser_permutation"))
             order <- ser_permutation(order)
         .check_dist_perm(x, order)
     }
-    
-    ## check dist (most C code only works with lower-triangle version) 
+
+    ## check dist (most C code only works with lower-triangle version)
     if(attr(x, "Diag") || attr(x, "Upper"))
         x <- as.dist(x, diag = FALSE, upper = FALSE)
     if(!is.double(x)) mode(x) <- "double"
 
     ## get methods
-    if(is.null(method)) method <- list_criterion_methods("dist") 
+    if(is.null(method)) method <- list_criterion_methods("dist")
     method <- lapply(method, function(m) get_criterion_method("dist", m))
-    
-    sapply(method, 
+
+    sapply(method,
         function(m) structure(m$fun(x, order, ...), names=m$name))
 }
 
@@ -56,20 +56,25 @@ criterion_path_length <- function(x, order = NULL, ...) {
     .Call("order_length", x, order, PACKAGE = "seriation")
 }
 
+criterion_lazy_path_length <- function(x, order = NULL, ...) {
+  if (is.null(order)) order <- 1:attr(x, "Size")
+  else order <- get_order(order)
+  .Call("lazy_path_length", x, order, PACKAGE = "seriation")
+}
 
-## Least squares criterion. measures the difference between the 
+## Least squares criterion. measures the difference between the
 ## dissimilarities between two elements and the rank distance
 ## (PermutMatrix).
 
 criterion_least_squares <- function(x, order = NULL, ...) {
-    if(is.null(order)) order <- 1:attr(x, "Size") 
+    if(is.null(order)) order <- 1:attr(x, "Size")
     else order <- get_order(order)
     .Call("least_squares_criterion", x, order, PACKAGE = "seriation")
 }
 
 ## inertia around the diagonal (see PermutMatrix)
 criterion_inertia <- function(x, order = NULL, ...) {
-    if(is.null(order)) order <- 1:attr(x, "Size") 
+    if(is.null(order)) order <- 1:attr(x, "Size")
     else order <- get_order(order)
     .Call("inertia_criterion", x, order, PACKAGE = "seriation")
 }
@@ -78,7 +83,7 @@ criterion_inertia <- function(x, order = NULL, ...) {
 ## 2002)
 ## method: 1...i, 2...s, 3...w
 .ar <- function(x, order = NULL, method = 1L) {
-    if(is.null(order)) order <- 1:attr(x, "Size") 
+    if(is.null(order)) order <- 1:attr(x, "Size")
     else order <- get_order(order)
     .Call("ar", x, order, as.integer(method), PACKAGE = "seriation")
 }
@@ -87,13 +92,35 @@ criterion_ar_events <- function(x, order, ...) .ar(x, order, 1L)
 criterion_ar_deviations <- function(x, order, ...) .ar(x, order, 2L)
 #criterion_ar_weighted <- function(x, order, ...) .ar(x, order, 3L)
 
-criterion_rgar <- function(x, order, w=NULL, ...) {
-  if(is.null(order)) order <- 1:attr(x, "Size") 
+## w \in [2,n-1]
+## or pct \in [0 and 100%]; 0 -> 2 and 100 -> n-1
+criterion_rgar <- function(x, order, w=NULL, pct=NULL, relative=TRUE, ...) {
+  if(is.null(order)) order <- 1:attr(x, "Size")
   else order <- get_order(order)
-  if(is.null(w)) w <- length(order)-1L
-  if(w<1 || w>=length(order)) stop("Window w needs to be 1<=w<length(order)!")
-  .Call("rgar", x, order, as.integer(w)+1L, PACKAGE = "seriation")
-} 
+
+  ### default is to take all
+  if(is.null(w) && is.null(pct)) w <- length(order)-1L
+  else if(is.null(w)) {
+      w <- floor((length(order)-3L)*pct/100)+2L
+      if(w<1) w <- 1
+  }
+
+  if(w<2 || w>=length(order)) stop("Window w needs to be 2<=w<length(order)!")
+  .Call("rgar", x, order, as.integer(w), as.integer(relative),
+    PACKAGE = "seriation")
+}
+
+criterion_bar <- function(x, order, b=NULL, ...) {
+    if(is.null(order)) order <- 1:attr(x, "Size")
+    else order <- get_order(order)
+
+    ### we default to 1/5
+    if(is.null(b)) b <- max(1,floor(length(order)/5))
+
+    if(b<1 || b >=length(order))
+      stop("Band size needs to be 1 <= b < length(order)!")
+    .Call("bar", x, order, as.integer(b), PACKAGE = "seriation")
+}
 
 criterion_gradient_raw <- function(x, order, ...) {
     if(is.null(order)) order <- 1:attr(x, "Size")
@@ -113,7 +140,7 @@ criterion_gradient_weighted <- function(x, order, ...) {
 criterion_2SUM <- function(x, order, ...) {
     if(is.null(order)) order <- 1:attr(x, "Size")
     else order <- get_order(order)
-    
+
     qap::qap.obj(.A_2SUM(attr(x, "Size")), 1/(1+as.matrix(x)), order)
 }
 
@@ -124,7 +151,7 @@ criterion_2SUM <- function(x, order, ...) {
 criterion_LS <- function(x, order, ...) {
     if(is.null(order)) order <- 1:attr(x, "Size")
     else order <- get_order(order)
-    
+
     qap::qap.obj(.A_LS(attr(x, "Size")), as.matrix(x), order)
 }
 
@@ -140,29 +167,39 @@ criterion_Neumann_stress_dist  <- function(x, order, ...)
 
 
 ### register methods
-set_criterion_method("dist", "AR_events" , criterion_ar_events, 
+set_criterion_method("dist", "AR_events" , criterion_ar_events,
     "Anti-Robinson events", FALSE)
 set_criterion_method("dist", "AR_deviations", criterion_ar_deviations,
     "Anti-Robinson deviations", FALSE)
 ## set_criterion_method("dist", "AR_weighted", criterion_ar_weighted)
+
 set_criterion_method("dist", "RGAR", criterion_rgar,
   "Relative generalized anti-Robinson events", FALSE)
+set_criterion_method("dist", "BAR", criterion_bar,
+  "Banded Anti-Robinson Form", FALSE)
+
 set_criterion_method("dist", "Gradient_raw" , criterion_gradient_raw,
     "Gradient measure", TRUE)
 set_criterion_method("dist", "Gradient_weighted", criterion_gradient_weighted,
     "Gradient measure (weighted)", TRUE)
+
 set_criterion_method("dist", "Path_length", criterion_path_length,
     "Hamiltonian path length", FALSE)
+set_criterion_method("dist", "Lazy_path_length", criterion_lazy_path_length,
+    "Lazy path length", FALSE)
+
 set_criterion_method("dist", "Inertia", criterion_inertia,
     "Inertia criterion", TRUE)
 set_criterion_method("dist", "Least_squares", criterion_least_squares,
     "Least squares criterion", FALSE)
 set_criterion_method("dist", "ME", criterion_ME_dist,
     "Measure of effectiveness", TRUE)
+
 set_criterion_method("dist", "Moore_stress", criterion_Moore_stress_dist,
     "Stress (Moore neighborhood)", FALSE)
 set_criterion_method("dist", "Neumann_stress", criterion_Neumann_stress_dist,
     "Stress (Neumann neighborhood)", FALSE)
+
 set_criterion_method("dist", "2SUM", criterion_2SUM,
     "2-SUM objective value (QAP)", FALSE)
 set_criterion_method("dist", "LS", criterion_LS,
